var currentScore = null;
var alphaTab1 = null;

function callNative(method, data) {
    try {
        console.log(method)
        android.callNative(method, JSON.stringify(data))
    } catch(error) {
        console.log(error);
    }
}

function base64ToArrayBuffer(base64) {
    var binary_string =  window.atob(base64);
    var len = binary_string.length;
    var bytes = new Uint8Array( len );
    for (var i = 0; i < len; i++)        {
        bytes[i] = binary_string.charCodeAt(i);
    }
    return bytes.buffer;
}

function b64int(n) {
    return n < 26 ? n + 65
    : n < 52 ? n + 71
    : n < 62 ? n - 4
    : n === 62 ? 43
    : n === 63 ? 47
    : 65
}

function bufferToBase64(input) {
    if (!(input instanceof Uint8Array)) {
        input = new Uint8Array(input.buffer)
    }
    
    var length = input.length
    var output = ""
    
    for (var value = 0, idx = 0; idx < length; idx++) {
        var bit = idx % 3
        
        value |= input[idx] << (16 >>> bit & 24)
        
        if (bit === 2 || input.length - idx === 1) {
            output += String.fromCharCode(
                                          b64int(value >>> 18 & 63)
                                          , b64int(value >>> 12 & 63)
                                          , b64int(value >>> 6 & 63)
                                          , b64int(value & 63)
                                          )
            value = 0
        }
    }
    
    return output.replace(/A(?=A$|$)/g, "=")
}

function loadTabForPrint(tabBase64, encoding="utf-8", showStave=true, width = null) {
    var a4 = document.createElement("div");
    if(!(width == null || width.length == 0)) {
        a4.style.width = width;
    } else {
        a4.style.width = "210mm";
    }
    document.body.appendChild(a4);
    var dualScreenLeft;
    var o = window["ScreenLeft"];
    if(typeof(o) != "undefined") {
        dualScreenLeft = window["ScreenLeft"];
    } else {
        dualScreenLeft = window.screen.left;
    }
    var dualScreenTop;
    var o1 = window["ScreenTop"];
    if(typeof(o1) != "undefined") {
        dualScreenTop = window["ScreenTop"];
    } else {
        dualScreenTop = window.screen.top;
    }
    var screenWidth;
    var o2 = window.innerWidth;
    if(typeof(o2) != "undefined") {
        screenWidth = window.innerWidth;
    } else {
        var o3 = window.document.documentElement.clientWidth;
        if(typeof(o3) != "undefined") {
            screenWidth = window.document.documentElement.clientWidth;
        } else {
            screenWidth = window.screen.width;
        }
    }
    var screenHeight;
    var o4 = window.innerHeight;
    if(typeof(o4) != "undefined") {
        screenHeight = window.innerHeight;
    } else {
        var o5 = window.document.documentElement.clientHeight;
        if(typeof(o5) != "undefined") {
            screenHeight = window.document.documentElement.clientHeight;
        } else {
            screenHeight = window.screen.height;
        }
    }
    var w = a4.offsetWidth + 50;
    var h = window.innerHeight;
    var left = (screenWidth / 2 | 0) - (w / 2 | 0) + dualScreenLeft;
    var top = (screenHeight / 2 | 0) - (h / 2 | 0) + dualScreenTop;
    window.resizeTo(w,h);
    window.moveTo(left,top);
    window.focus();
    var settings = alphaTab.Settings.get_Defaults();
	settings.FontDirectory = "js/Font/"
    settings.Scale = 0.8;
    settings.StretchForce = 0.8;
    settings.DisableLazyLoading = true;
    settings.UseWorkers = false;
    settings.ImporterSettings["encoding"] = encoding;
    settings.Layout.AdditionalSettings["hideChordDiagram"] = true;
    if (!showStave) {
        settings.Staves.AdditionalSettings.rhythm = true;
        settings.Staves.Id = "tab"
    }
    alphaTab1 = new alphaTab.platform.javaScript.AlphaTabApi(a4,settings);
    var file = base64ToArrayBuffer(tabBase64);
    currentScore = alphaTab.importer.ScoreLoader.LoadScoreFromBytes(new Uint8Array(file), settings);
    var reg = /^((https|http|ftp|rtsp|mms){0,1}(:\/\/){0,1})www\.(([A-Za-z0-9-~]+)\.)+([A-Za-z0-9-~\/])+$/;
    if(reg.test(currentScore.SubTitle)) {
        currentScore.SubTitle = "";
    }
    callNative("scoreLoaded", createTrackInfo(currentScore));
    alphaTab1.Renderer.add_PostRenderFinished(function() {
        alphaTab1.CanvasElement.set_Height(-1);
        callNative("renderFinished", "")
    });
    callNative("startRending", "");
    alphaTab1.RenderTracks(currentScore,[0],true);
}

function setTracksToPrint(tracks) {
    console.log(tracks)
    callNative("startRending", "");
    alphaTab1.RenderTracks(currentScore,tracks,true);
}

function createTrackInfo(score) {
    console.log(score)
    var trackInfos = [];
    for(var i = 0; i < score.Tracks.length; i++) {
        trackInfos[i] = {
            index: score.Tracks[i].Index,
            name: score.Tracks[i].Name,
            solo: score.Tracks[i].PlaybackInfo.IsSolo,
            mute: score.Tracks[i].PlaybackInfo.IsMute,
            volume: score.Tracks[i].PlaybackInfo.Volume,
            inst: score.Tracks[i].PlaybackInfo.Program,
            channel0: score.Tracks[i].PlaybackInfo.PrimaryChannel,
            channel1: score.Tracks[i].PlaybackInfo.SecondaryChannel
        }
    }
    console.log(trackInfos)
    return trackInfos;
}

//window.onerror = handleError;
function handleError(msg,url,l){
    console.log("handle Error");
    console.log(msg);
    console.log(l);
    callNative("loadScoreError", msg);
    return true
}

function scoreInfo(score) {
    return {
    album: score.Album,
    artist: score.Artist,
    copyright: score.Copyright,
    instructions: score.Instructions,
    music: score.Music,
    notices: score.Notices,
    subtitle: score.Subtitle,
    tab: score.Tab,
    tempo: score.Tempo,
    tempoLabel: score.TempoLabel,
    title: score.Title
    }
}

$(document).ready(function() {
    console.log("ready")
    callNative("documentReady", "");
});